#include "etsi_ts103759_data_codec.hh"

#include "loggers.hh"

int etsi_ts103759_data_codec::encode(const EtsiTs103759Core::EtsiTs103759Data &p_etsi_ts_103759_data, OCTETSTRING &p_data) {
  loggers::get_instance().log(">>> etsi_ts103759_data_codec::encode: %s", p_etsi_ts_103759_data.get_descriptor()->name);

  TTCN_EncDec::clear_error();
  TTCN_Buffer buffer;
  p_etsi_ts_103759_data.encode(*p_etsi_ts_103759_data.get_descriptor(), buffer, TTCN_EncDec::CT_OER);
  p_data = OCTETSTRING(buffer.get_len(), buffer.get_data());
  loggers::get_instance().log_msg("etsi_ts103759_data_codec::encode: ", p_data);

  return 0;
}

int etsi_ts103759_data_codec::decode(const OCTETSTRING &p_data, EtsiTs103759Core::EtsiTs103759Data &p_etsi_ts_103759_data, params *p_params) {
  loggers::get_instance().log_msg(">>> etsi_ts103759_data_codec::decode: ", p_data);

  TTCN_EncDec::clear_error();
  TTCN_Buffer decoding_buffer(p_data);
  //  _params = params;
  p_etsi_ts_103759_data.decode(*p_etsi_ts_103759_data.get_descriptor(), decoding_buffer, TTCN_EncDec::CT_OER);

  /**
   * FIXME FSCOM Bug in TITAN TTCN-3 and ASN.1 Compiler for the TTCN-3 Test Executor, version 9.0.0
   */
  Ieee1609Dot2::CertificateBase* cert = nullptr;
  if (p_etsi_ts_103759_data.report().content().ischosen(EtsiTs103759Core::AidSpecificReport_content_type::ALT_asrCam)) {
    if (p_etsi_ts_103759_data.report().content().asrCam().v2xPduEvidence()[0].certificate().is_present()) {
      cert = static_cast<Ieee1609Dot2::CertificateBase*>(p_etsi_ts_103759_data.report().content().asrCam().v2xPduEvidence()[0].certificate().get_opt_value());
    }
  /*} else if (p_etsi_ts_103759_data.report().content().ischosen(EtsiTs103759Core::AidSpecificReport_content_type::ALT_asrDenm)) {
    if (p_etsi_ts_103759_data.report().content().asrDenm().v2xPduEvidence()[0].certificate().is_present()) {
      cert = static_cast<Ieee1609Dot2::CertificateBase*>(p_etsi_ts_103759_data.report().content().asrCam().v2xPduEvidence()[0].certificate().get_opt_value());
    }*/
  }
  if (cert != nullptr) {
    cert->toBeSigned().appExtensions().set_to_omit();
    cert->toBeSigned().certIssueExtensions().set_to_omit();
    cert->toBeSigned().certRequestExtension().set_to_omit();
  }
  /**
   * 
   */

  loggers::get_instance().log_msg("<<< etsi_ts103759_data_codec::decode: ", (const Base_Type &)p_etsi_ts_103759_data);
  return 0;
}
