#include "etsi_ts102941_data_codec.hh"

#include "loggers.hh"

int etsi_ts102941_data_codec::encode(const EtsiTs102941MessagesCa::EtsiTs102941Data &p_etsi_ts_102941_data, OCTETSTRING &p_data) {
  loggers::get_instance().log(">>> etsi_ts102941_data_codec::encode: %s", p_etsi_ts_102941_data.get_descriptor()->name);

  TTCN_EncDec::clear_error();
  TTCN_Buffer buffer;
  p_etsi_ts_102941_data.encode(*p_etsi_ts_102941_data.get_descriptor(), buffer, TTCN_EncDec::CT_OER);
  p_data = OCTETSTRING(buffer.get_len(), buffer.get_data());
  loggers::get_instance().log_msg("etsi_ts102941_data_codec::encode: ", p_data);

  return 0;
}

int etsi_ts102941_data_codec::decode(const OCTETSTRING &p_data, EtsiTs102941MessagesCa::EtsiTs102941Data &p_etsi_ts_102941_data, params *p_params) {
  loggers::get_instance().log_msg(">>> etsi_ts102941_data_codec::decode: ", p_data);

  TTCN_EncDec::clear_error();
  TTCN_Buffer decoding_buffer(p_data);
  //  _params = params;
  p_etsi_ts_102941_data.decode(*p_etsi_ts_102941_data.get_descriptor(), decoding_buffer, TTCN_EncDec::CT_OER);

  /**
   * FIXME FSCOM Bug in TITAN TTCN-3 and ASN.1 Compiler for the TTCN-3 Test Executor, version 9.0.0
   */
  if (p_etsi_ts_102941_data.content().ischosen(EtsiTs102941MessagesCa::EtsiTs102941DataContent::ALT_enrolmentResponse)) {
    if (p_etsi_ts_102941_data.content().enrolmentResponse().certificate().ispresent()) {
      Ieee1609Dot2::CertificateBase* cert = static_cast<Ieee1609Dot2::CertificateBase*>(p_etsi_ts_102941_data.content().enrolmentResponse().certificate().get_opt_value());
      cert->toBeSigned().appExtensions().set_to_omit();
      cert->toBeSigned().certIssueExtensions().set_to_omit();
      cert->toBeSigned().certRequestExtension().set_to_omit();
    }
  } else if (p_etsi_ts_102941_data.content().ischosen(EtsiTs102941MessagesCa::EtsiTs102941DataContent::ALT_butterflyAuthorizationRequest)) {
    p_etsi_ts_102941_data.content().butterflyAuthorizationRequest().tbsCert().appExtensions().set_to_omit();
    p_etsi_ts_102941_data.content().butterflyAuthorizationRequest().tbsCert().certIssueExtensions().set_to_omit();
    p_etsi_ts_102941_data.content().butterflyAuthorizationRequest().tbsCert().certRequestExtension().set_to_omit();
  }
  /**
   * 
   */

  loggers::get_instance().log_msg("<<< etsi_ts102941_data_codec::decode: ", (const Base_Type &)p_etsi_ts_102941_data);
  return 0;
}
