#include "EtsiTs103097Module.hh"

#include "asn1/EtsiTs103097Certificate.h"
#include "asn1/asn_application.h" // from asn1c
#include "etsi_ts103097_certificate_codec.hh"

#include "loggers.hh"

int etsi_ts103097_certificate_codec::encode(const Ieee1609Dot2::CertificateBase &p_cert, OCTETSTRING &p_data) {
  loggers::get_instance().log(">>> etsi_ts103097_certificate_codec::encode: %s", p_cert.get_descriptor()->name);

  TTCN_EncDec::clear_error();
  TTCN_Buffer buffer;
  p_cert.encode(*p_cert.get_descriptor(), buffer, TTCN_EncDec::CT_OER);
  p_data = OCTETSTRING(buffer.get_len(), buffer.get_data());
  loggers::get_instance().log_msg("etsi_ts103097_certificate_codec::encode: ", p_data);

  return 0;
}

int etsi_ts103097_certificate_codec::decode(const OCTETSTRING &p_data, Ieee1609Dot2::CertificateBase &p_cert, params *params) {
  loggers::get_instance().log_msg(">>> etsi_ts103097_certificate_codec::decode: ", p_data);

  TTCN_EncDec::clear_error();
  TTCN_Buffer decoding_buffer(p_data);
  _params = params;
  p_cert.decode(*p_cert.get_descriptor(), decoding_buffer, TTCN_EncDec::CT_OER);

  /**
   * FIXME FSCOM Bug in TITAN TTCN-3 and ASN.1 Compiler for the TTCN-3 Test Executor, version 9.0.0
   */
  p_cert.toBeSigned().appExtensions().set_to_omit();
  p_cert.toBeSigned().certIssueExtensions().set_to_omit();
  p_cert.toBeSigned().certRequestExtension().set_to_omit();
  /**
   * 
   */

  loggers::get_instance().log_msg("<<< etsi_ts103097_certificate_codec::decode: ", (const Base_Type &)p_cert);
  return 0;
}
