#include <zip.h>

#include "http_etsi_ieee1609dot2dot1_codec.hh"

#include "LibHttp_BinaryMessageBodyTypes.hh"

#include "loggers.hh"

int http_etsi_ieee1609dot2dot1_codec::decode(const OCTETSTRING &p_data, LibHttp__BinaryMessageBodyTypes::BfkZipFile &p_bfk_zip_file, params *p_params) {
  loggers::get_instance().log(">>> http_etsi_ieee1609dot2dot1_codec::decode: Butterfly AT download response corresponding to a ZIP file: Version: %02x.%02x", p_data[4].get_octet(), p_data[5].get_octet());
  loggers::get_instance().log(">>> http_etsi_ieee1609dot2dot1_codec::decode: Butterfly AT download response corresponding to a ZIP file: Compression method: %02x.%02x", p_data[8].get_octet(), p_data[9].get_octet());
  // Unzip header file
  p_bfk_zip_file.identifier() = substr(p_data, 0, 4);
  p_bfk_zip_file.version() = substr(p_data, 4, 2);
  p_bfk_zip_file.flags() = substr(p_data, 6, 2);
  p_bfk_zip_file.compression() = substr(p_data, 8, 2);
  p_bfk_zip_file.content().raw() = substr(p_data, 10, p_data.lengthof() - 10 - 1);

  zip_error_t error;
  zip_source_t *src = nullptr;
  zip_stat_t zst;
  zip_t *za = nullptr;
  int ret_code = -1;
  LibHttp__BinaryMessageBodyTypes::BfkZipFileContent_signed__messages signed_messages;
  ::zip_error_init(&error);
  if ((src = ::zip_source_buffer_create(static_cast<const unsigned char*>(p_data), p_data.lengthof(), 1, &error)) == nullptr) {
    loggers::get_instance().error("http_etsi_ieee1609dot2dot1_codec::decode: ::zip_source_buffer_create failure: %s", ::zip_error_strerror(&error));
    ::zip_error_fini(&error);
    return -1;
  }
  if (::zip_source_stat(src, &zst) < 0) {
    loggers::get_instance().error("http_etsi_ieee1609dot2dot1_codec::decode: ::zip_source_stat failure: %s\n", ::zip_error_strerror(::zip_source_error(src)));
    goto decode_cleanup;
  }
  loggers::get_instance().log("http_etsi_ieee1609dot2dot1_codec::decode: zst.valid: %08x", zst.valid);
  loggers::get_instance().log("http_etsi_ieee1609dot2dot1_codec::decode: zst.valid: %p", zst.name);
  loggers::get_instance().log("http_etsi_ieee1609dot2dot1_codec::decode: zst.size: %ld", zst.size);
  loggers::get_instance().log("http_etsi_ieee1609dot2dot1_codec: decode: mtime: %u", (unsigned int)zst.mtime);
  loggers::get_instance().log("http_etsi_ieee1609dot2dot1_codec::decode: zst.comp_size: %ld", zst.comp_size);
  loggers::get_instance().log("http_etsi_ieee1609dot2dot1_codec::decode: zst.crc: %04x", zst.crc);
  loggers::get_instance().log("http_etsi_ieee1609dot2dot1_codec::decode: zst.encryption_method: %02x", zst.encryption_method);

  if (::zip_source_open(src) < 0) {
    loggers::get_instance().error("http_etsi_ieee1609dot2dot1_codec::decode: ::zip_source_open failure: %s", ::zip_error_strerror(::zip_source_error(src)));
    goto decode_cleanup;
  }
  
  if ((za = ::zip_open_from_source(src, 0, &error)) == NULL) {
    loggers::get_instance().error("http_etsi_ieee1609dot2dot1_codec::decode: Can't open zip from source: %s\n", ::zip_error_strerror(&error));
    goto decode_cleanup;
  }
  ::zip_error_fini(&error);

  loggers::get_instance().log("http_etsi_ieee1609dot2dot1_codec:: Number of entries: %d", ::zip_get_num_entries(za, 0));
  signed_messages.set_size(::zip_get_num_entries(za, 0));
  for (int i = 0; i < ::zip_get_num_entries(za, 0); i++) {
    loggers::get_instance().log("http_etsi_ieee1609dot2dot1_codec::decode: Processing entry #%d", i);
    struct zip_stat sb;
    if (::zip_stat_index(za, i, 0, &sb) == 0) {
      loggers::get_instance().log("http_etsi_ieee1609dot2dot1_codec::decode:==================");
      loggers::get_instance().log("http_etsi_ieee1609dot2dot1_codec::decode: Name: [%s], ", sb.name);
      loggers::get_instance().log("http_etsi_ieee1609dot2dot1_codec::decode: Size: [%llu], ", sb.size);
      loggers::get_instance().log("http_etsi_ieee1609dot2dot1_codec::decode: mtime: [%u]", (unsigned int)sb.mtime);
      struct zip_file *zf = ::zip_fopen_index(za, i, 0);
      if (zf == nullptr) {
        loggers::get_instance().error("http_etsi_ieee1609dot2dot1_codec::decode: ::zip_fopen_index failure");
        goto decode_cleanup;
      }
      unsigned char* buffer = (unsigned char*)new unsigned char*[sb.size];
      size_t len = ::zip_fread(zf, buffer, sb.size);
      if (len < 0) {
        loggers::get_instance().error("http_etsi_ieee1609dot2dot1_codec::decode: ::zip_fread failure");
        delete [] buffer;
        ::zip_fclose(zf);
        goto decode_cleanup;
      }
      OCTETSTRING os;
      loggers::get_instance().log_to_hexa("http_etsi_ieee1609dot2dot1_codec::decode: content:", buffer, sb.size);
      if (*buffer == 0x81) { // Remove 0x81
        os = OCTETSTRING(sb.size - 1, buffer + 1);
      } else {
        os = OCTETSTRING(sb.size, buffer);
      }
      loggers::get_instance().log_to_hexa("http_etsi_ieee1609dot2dot1_codec::decode: content:", os);
      Ieee1609Dot2::Ieee1609Dot2Data ieee1609Dot2Data;
      if (_codec.decode(os, ieee1609Dot2Data, p_params) == -1) {
        loggers::get_instance().error("http_etsi_ieee1609dot2dot1_codec::decode: Failed to decode ieee1609Dot2Data");
        delete [] buffer;
        ::zip_fclose(zf);
        goto decode_cleanup;
      }
      signed_messages[i] = ieee1609Dot2Data;
      delete [] buffer;
      ::zip_fclose(zf);
    }
  } // End of 'for' statement
  loggers::get_instance().log_msg("http_etsi_ieee1609dot2dot1_codec::decode: After loop: signed_messages=", signed_messages);
  //::zip_source_keep(src);
  // Close the archive
  //::zip_close(za);
  // Close the source
  p_bfk_zip_file.content().signed__messages() = signed_messages;
  ret_code = 0; // Everythig goes well
decode_cleanup:

  ::zip_source_close(src);
  //::zip_source_free(src);

  loggers::get_instance().log_msg("http_etsi_ieee1609dot2dot1_codec::decode: p_bfk_zip_file=", p_bfk_zip_file);
  loggers::get_instance().log("<<< http_etsi_ieee1609dot2dot1_codec::decode: %d", ret_code);
  return ret_code;
}
