#!/usr/bin/env bash

set -euo pipefail
set +vx

usage() {
	echo "No command specified. Available commands:"
	for i in $(echo $commands | sed "s/,/ /g")
	do
		echo "  $i"
	done
	echo "For build,clean,list,modulepar,rebuild,build-run and run, ATS identifier need to be specified (e.g. CAM, DENM, Security, Pki, MBR...)"
	echo "e.g. docker-run.sh build Security # To build the AtsSecurity test suite"
}

cli="${GEN_DIR}/bin/TS.ITS_r2"

commands="help,build,clean,ip,list,modulepar,ping,rebuild,build-run,run,version"

if [ $# == 0 ]; then
  usage
	exit 1
fi

if [ $# == 2 ]; then 
  if [ "$2" != "" ]; then
    export ATS=Ats$2
    cli="${GEN_DIR}/bin/$ATS"
    echo "cli: $cli"
  fi
fi


case "$1" in
  build)
    cd ${GEN_DIR} && make
    ;;
  clean)
    cd ${GEN_DIR} && make clean
    ;;
  ip)
    ip address
    ;;
  list)
    ${cli} -l
    ;;
  modulepar)
    ${cli} -p
    ;;
  ping)
    sudo ping $2
    ;;
  rebuild)
    cd ${GEN_DIR} && rm -fr ./bin ./build ; make
    ;;
  build-run)
    d ${GEN_DIR}/scripts && ./run_all.bash 0
    ;;
  run)
    cd ${GEN_DIR}/scripts && ./run_all.bash
    ;;
  version)
    compiler -v
    ;;
  *)
    usage
    exit 1
    ;;
esac

unset ATS
exit 0
